-- vtop.vhd
-- -------------------------------------------------------------------------------
-- --  uC Calculator with keyboard input and LED output --
-- -------------------------------------------------------------------------------
-- Programers :  Derwyn Hollobaugh and Anthony Dotterer
--
-- emails:	wolfboydj@hotmail.com ( Derwyn ), whatupgee@hotmail.com ( Anthony )
--
-- Purpose : The uC Calculator is a simple calculator that takes a keyboard as an input
--		and a single LED as its output.  Unforunatly due to the limited 
--		output devices, we can only have this calculator use 0-9 as numbers
--		and 0-F in hex as outputs.  The inputs from the keyboard are as
--		follows (Note: do not use Capslocks and/or Shift):
--
--		0-9 = The numbers across the top of the keyboard (not the numpad)
--		p    = The letter 'p' is an operation that if hit in the first operation
--				space will add both numbers entered
--		s    = The letter 's' is the second type of operation that should be used
--				in the first operation space. It will subtract the second
--				number entered from the first number entered
--		m   = The letter 'm' is the last operation in the uC.  It should be hit
--				in the first operation space.  It will multiply the numbers
--				entered together
--		=   = The '=' key is a dummby operation that should be used a the end of
--				of the total operation.  If used as the first operation it
--				will trigger it to add
--
--	The order of keys to be entered will follow the following configuration (Note: extra numbers
--		pressed in the operand spaces will replace the previous number entered) :
--
--		1st keystroke : first operand
--		2nd keystroke : first operation (addition, subtraction, and multiplcation)
--		3rd keystroke : second operand
--		4th keystroke : the evaluation ( '=' expected, but any operation will trigger it)
--				 
-- --------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

-- --------------------------------------------
Entity uCcalculator is
-- --------------------------------------------
    Port (   CLK : In    std_logic;	-- The clock provided by the FPGA
	 key_clk : In	 std_logic;	-- The clock from the keyboard
	key_data : In	 std_logic;	-- The data line from the keyboard
           nPSEN : In    std_logic;	-- uC Program-Store Enable
            PC_D : In    std_logic_vector (5 downto 0);
					-- The data from the parallel cable
             nWR : In    std_logic;	-- uC Active-Low Write (also controls RAM)
             ALE : In    std_logic;	-- uC Active-High Address Latch Enable
              AH : In   std_logic_vector (15 downto 8);					
              AD : InOut std_logic_vector (7 downto 0);					
              AL : Out  std_logic_vector  (7 downto 0);					
             nCE : Out   std_logic;	-- Active-Low Chip Enable
             nRD : In    std_logic;	-- uC Active-Low Read
             nOE : Out   std_logic;	-- Active-Low Output Enable
           AH_16 : Out   std_logic;	-- Upper 64 k Enable
             RST : Out   std_logic;	-- uC Active-High Reset
           XTAL1 : Out   std_logic;	-- uC Clock
             LED : Out   std_logic_vector (6 downto 0)
					-- The bit configuration out to the LED display
            );
end uCcalculator;

-- --------------------------------------------
Architecture Behavior of uCcalculator is
-- --------------------------------------------

-- ------------------------------------------------------------
-- The following components are registers that will be used by
-- 	the uC
-- ------------------------------------------------------------

component ScanReg is
	port (  kbclk, kbdata, reset: in std_logic;
		scancode: out std_logic_vector(7 downto 0);
		done: out std_logic);
end component;

component Reg is
	generic( N : positive := 8 );
	port( 	D : in std_logic_vector( N-1 downto 0);
	  	C, Clk, R : in std_logic;
	      	Q : out std_logic_vector(N-1 downto 0));
end component;

-- -------------------------------------------------------------
-- The following converters will convert the keyboard input to 
--	the design into usable data, and convert the output from 
--	data into LED format
-- -------------------------------------------------------------

component scanconverter is
	port (scancode: in std_logic_vector(7 downto 0);
		kind: out std_logic;
		digit: out std_logic_vector(3 downto 0));
end component;

component hex2seven is
	port(hex: in std_logic_vector(3 downto 0);
	     seg: out std_logic_vector(6 downto 0));
end component;

-- --------------------------------------------------------------
-- The following components will take a signal and either make
--	it clearer or make it a clock signal
-- --------------------------------------------------------------

component ibuf is	-- This component will "BEEF" a signal
	port(i: in std_logic;
		 o: out std_logic);
end component;

component bufg is	-- This component will make a signal become a clock signal
	port(i: in std_logic;
		 o: out std_logic);
end component;

-- --------------------------------------------------------------
-- The following are varies signals that connect with in this 
--	design
-- --------------------------------------------------------------

signal    ENLED : std_logic;  -- This signal will enable the loading of the LED reg
signal      DOE : std_logic;  -- This signal will disable the Tri-State inorder
			      --	for the uC to Read Dout
signal     RSTi : std_logic;  -- This signal connects the LSB of the parallel
			      -- 	cable to the Reset of the uC
signal      Din : std_logic_vector (7 downto 0);  
			      -- The MUXED output of AD
signal     Dout : std_logic_vector (7 downto 0);
			      -- The TRI-stated Input to AD
signal       WR : std_logic;  -- This signal determines a writing operation 
			      --	(will only be used by LEDReg in this design)
signal       RD : std_logic;  -- This signal determines a reading operation
			      -- 	(will only be used by ScanReg in this design)

signal Out_temp : std_logic_vector( 7 downto 0 );  
			      -- This signal is the output from the ScanReg containing
			      --	the scancode
signal buf_key_clk : std_logic;  
			      -- This signal is the output from the key_clk buffer
signal    kbclk : std_logic;  -- This signal is the output from the key_clk clock buffer
signal   kbdata : std_logic;  -- This signal is the output from the key_data buffer
signal     ttype : std_logic;  -- This signal repersents the input is an operation
signal     done : std_logic;  -- This signal tells the uC when to input data
signal Hex_out, Hex_in : std_logic_vector( 3 downto 0 );

begin

-- ---------------------------------------------------------------
-- This Section sets the controls sent from the uC to ineract with
--	the registers in the design as well as the data sent back
-- ---------------------------------------------------------------

	AH_16 <= '0';     -- Make sure we work on the lowest 64k Ram

	RSTi <= PC_D(0);  -- this attaches the lower bit on the parallel cable to
			  --	the uC to control the reset 

	XTAL1 <= CLK;     -- pass the clock thru to the uC

	nCE <= AH(15);		-- This is attached to the Chip Enable on the RAM chip.  This will have the Ram disable it-
						--	self so that it will not confict with the Data that might be on the lines. 
	nOE <= not (not nRD or not nPSEN);
				-- Output enable for the Ram chip

	WR   <= not nWR;  -- Inverts the Active-low signal to Active-high
	RD   <= not nRD;  -- Inverts the Active-low signal to Active-high

	RST  <= RSTi;	  -- This connects the reset to the uC

	Dout <= "00" & done & ttype & Hex_in;
			  -- This is the signal to be sent to the uC when the uC request the ScanReg
			  --	In another design one may want multiple inputs to the uC, and then would
			  --	need a MUXing device as well as extra signals in the DECODE process to
			  --	configure the MUX
	
	Din <= AD;  -- FeedBack

	LATCH_ENABLE : process (CLK,RSTi)
		begin
  			if RSTi='1' then AL <= (others=>'0');
  			elsif rising_edge(CLK) then
    				if ALE='1' then	AL <= Din; end if;
  			end if;
		end process;
		
	TRI_STATE : process (AD, Dout, DOE)  -- TriState output
		begin
  			if DOE='1' then AD <= Dout;
  			else AD <= (others=>'Z');
  			end if;
		end process;

-- ------------------------------------------------------------
-- This next section connects the proper signals to the correct
-- 	output components
-- ------------------------------------------------------------

 	LEDReg: Reg generic map (4) 
		port map ( 	Din(3 downto 0), -- The lower byte of the uC output
				ENLED,		 -- The enable signal
				CLK, RSTi,        -- Clock and Reset signals
				Hex_out		 -- The registers byte output
		);

	LEDCov: hex2seven port map (
				Hex_out,  -- The byte output from the LEDReg
				LED	  -- The converted output bit placement
		);

-- ------------------------------------------------------------
-- This section buffers the clk and data from the keyboard
-- ------------------------------------------------------------

	Buffer_dclk : ibuf port map (
				key_data,  -- The weak signal from the keyboard
				kbdata     -- a "BEEFY" signal
		);

	Buffer_kclk : ibuf port map (
				key_clk,     -- The weak signal from the keyboard
				buf_key_clk  -- a "BEEFY" signal
		);

	Clock_Buffer : bufg port map (
				buf_key_clk, -- a "BEEFY" signal
				kbclk	     -- The finial usable clock signal
		); 

-- -------------------------------------------------------------
-- This section connects the proper signals to the correct input
-- 	components
-- -------------------------------------------------------------

	Scan : ScanReg port map (
				kbclk,	  -- The usable clock from keyboard
				kbdata,   -- The data from the keyboard
				rsti,    -- Universal reset
				Out_temp, -- The output from the ScanReg
				done	  -- Deterimes if input is valid
				);

	ScanConv : scanconverter port map (
				Out_temp, -- this is the Scancode
				ttype,		      -- determines operation
				Hex_in   	      -- digit repersentation
				);

-- -------------------------------------------------------------
-- The process will decode what the uC wants from the circuit
-- -------------------------------------------------------------				

	DECODE : process (AH,RD,WR)
		begin
  			DOE     <= '0';  -- This will reactivate the TRI-State

  			ENLED   <= '0';  -- This will tell the LEDReg to stop
					 -- 	Loading

  			if    (AH(15 downto 12)= x"F") then
    				ENLED <= WR;
					-- This is the uC's way of discribing what
					--	register it wants to target to write too

  			elsif (AH(15 downto 12)= x"E") then
    				DOE <= RD;     
					-- This is the uC's way of telling the circuit
					--	that it wants to read information from
					-- 	according register
    			end if;
		end process;





end Behavior;

